from datetime import datetime as dt

class Timer():    
    """    
    Timer class
    Declare a timer object that measure the time 
    of a running application
    """    

    def __init__(self, start_now = False):
        self.start_time = None
        self.stop_time = None
        self.duration = None
        if start_now:
            self.start()
            
    def __repr__(self):
        return f'\n started: {str(self.start_time)} \n finished:{str(self.stop_time)} \n duration:{str(self.duration)}'

    def start(self):
        if self.start_time==None:
            self.start_time = dt.datetime.now()
        else:
            print('Started at: {}'.format(self.start_time))
    
    def stop(self):
        if self.stop_time==None:
            self.stop_time = dt.datetime.now()
            self.duration = self.stop_time - self.start_time
        else:
            print('Stopped at: {}'.format(self.stop_time))
            
class Timers():
    """    
    Multi Timer class
    Dictionary with multiple timers to divide steps in a aplication
    and can be called without declaring keys
    """    

    def __init__(self, start_now = False, n=1,timers_names=None):
        self.timers = dict()
        self.__starts = 0 
        self.__stops = 0

    def __repr__(self):
        return f'{self.timers}'
        
    def create_timer(self,timer_name):
        self.timers[timer_name] = Timer()
    
    def start(self,timer_name=None):
        if timer_name == None:
            timer_name = self.__starts
            self.__starts+=1
        
        self.create_timer(timer_name)
        self.timers[timer_name].start()
            
    def stop(self,timer_name=None):
        if timer_name == None:
            timer_name = self.__stops
            self.timers[timer_name].stop()
            self.__stops+=1
            return
        else:
            self.timers[timer_name].stop()


from jh_utils.data.pandas import sql
import dask as dd

def migrate_table_pandas(query, table_name, engine_origin, engine_destiny, destiny_schema, if_exists):
    df = sql.get_sql_table(query, engine_origin)
    sql.write_table(df, table_name, destiny_schema, engine_destiny,
                    if_exists=if_exists, chunksize=10_000, index=False, close_connection=False)

def migrate_function_pandas(engine_origin, 
                         engine_destiny, 
                         destiny_schema,index=False):
    def output_func(query, table_name, if_exists='replace', chunksize=10_000):
        df = sql.get_sql_table(query, engine_origin)
        sql.write_table(df, table_name, destiny_schema, engine_destiny,
                        if_exists=if_exists, chunksize=10_000, 
                        index=False, close_connection=False)
    return output_func


def migrate_table_dask(table,
                       table_id,
                       input_schema,
                       output_schema,
                       uri_input,
                       uri_output,
                       npartitions,
                       bytes_per_chunk='256MB',
                       parallel=True,
                       if_exists='append',
                       method='multi'):
    df = dd.read_sql_table(table=table,
                           uri=uri_input,
                           schema=input_schema,
                           index_col=table_id,
                           npartitions=npartitions,
                           bytes_per_chunk=bytes_per_chunk)
    df = df.drop(f'{table_id}__1', axis=1)
    dd.to_sql(df,
              uri=uri_output,
              name=table,
              schema=output_schema,
              if_exists=if_exists,
              parallel=parallel,
              method=method)


def migrate_function_dask(uri_input, uri_output):
    def output_func(table, 
                    table_id, 
                    input_schema, 
                    output_schema, 
                    npartitions, 
                    bytes_per_chunk='256MB', 
                    parallel=True, 
                    if_exists='append', 
                    method='multi'):
        df = dd.read_sql_table(table=table,
                               uri=uri_input,
                               schema=input_schema,
                               index_col=table_id,
                               npartitions=npartitions,
                               bytes_per_chunk=bytes_per_chunk)
        df = df.drop(f'{table_id}__1', axis=1)
        dd.to_sql(df,
                  uri=uri_output,
                  name=table,
                  schema=output_schema,
                  if_exists=if_exists,
                  parallel=parallel,
                  method=method)
    return output_func



import os

def bash(command:str):
    os.system(command)

def create(name,is_folder=False):
    if is_folder==True:
        os.mkdir(name)
        return
    os.mkfifo(name)

def remove(name,is_folder=False):
    if is_folder==True:
        os.rmdir(name)
        return
    os.remove(name)

def ls(path:str='.', contains = [''], not_contains=[''],files=True, directories=True):
    ## just to transform string in list if necessary
    if type(contains)==str:
        contains = [contains]
    if type(contains)==str:
        not_contains = [not_contains]
    
    ## set first list
    if files==True and directories==True:
        final_ls = os.listdir(path)
    elif files==True:
        final_ls = list(filter(os.path.isfile, os.listdir(path)))
    elif directories==True:
        final_ls = list(filter(os.path.isdir, os.listdir(path)))
    else:
        return 'ERROR: DIRECTORIES OF FILES NEED TO BE TRUE'
    
    ## filter list
    for i in contains:
        final_ls = list(filter(lambda k: i in k, final_ls))
    for i in not_contains:
        final_ls = list(filter(lambda k: i in k, final_ls))

    return final_ls




import pandas as pd
import ast

def create_dimension(dataframe, column_name):
    """
    [summary]

    Args:
        dataframe (dataframe): [description]
        column_name (string): [description]

    Returns:
        dataframe: [description]
    """ 
    
    dimension = dataframe[column_name]
    dimension = pd.DataFrame(dimension.unique()).reset_index()
    dimension.columns = ['id'+column_name, column_name]
    return dimension

def create_bridge_dimension(dataframe,column_name, index_column):
    temp_df = dataframe[[column_name, index_column]]
    temp_df.iloc[:,0] = temp_df.iloc[:,0].fillna(value='[]').apply(ast.literal_eval)
    
    ## ! clean NA and explode the list
    temp_df = temp_df.explode(column_name)
    temp_df = temp_df[temp_df.iloc[:,0].isna() == False]
    
    ## ! create dimension
    dimension = create_dimension(temp_df,column_name)
    temp_df = temp_df.merge(dimension,on=column_name,how='left').iloc[:,1:]
    return (temp_df, dimension)

def modelate_database(dataframe, columns = [], bridge_columns = [], index_column = None):
    ret = dict()
    
    ## create the dimensions
    if columns != []:
        for i in columns:
            dim_temp = create_dimension(dataframe,i)
            ret[i] = dim_temp
            dataframe = dataframe.merge(dim_temp,on=i,how='left')

    if bridge_columns != []:
        if index_column == None:
            return "ERROR: Bridge tables need index column parameter"
        
        for i in bridge_columns:
            bridge_temp, dim_temp = create_bridge_dimension(dataframe,i, index_column)
            ret[i] = dim_temp
            ret['bridge_'+i] = bridge_temp

    ## add fact to return
    ret['fact'] = dataframe.drop(columns + bridge_columns,axis=1)
    return ret


import io
import os
import re

from setuptools import find_packages
from setuptools import setup

from dotenv import dotenv_values
env = dotenv_values('.env')

version = f'{env["major"]}.{env["minor"]}.{env["patch"]}'

with open("README.md", "r") as fh:
    long_description = fh.read()

def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding='utf-8') as fd:
        return re.sub(text_type(r':[a-z]+:`~?(.*?)`'), text_type(r'``\1``'), fd.read())

setup(
    name="jh_utils",
    version=version,
    url='https://github.com/JohnHolz/jh_utils',
    license='MIT',

    author="joao holz",
    author_email="joaopaulo.paivaholz@gmail.com",

    description="Some simple functions to all projects",
    long_description=long_description,
    long_description_content_type='text/markdown',

    packages=find_packages(exclude=('tests',)),

    install_requires=[],

    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
    ],
)



from sqlalchemy import create_engine
import psycopg2

def create_connection(database: str, user: str, password: str, host: str, port: str, sgbd = 'postgresql'):
    """
    Declare a db connection

    Args:
        database (str): database name
        user (str): user
        password (str): password
        host (str): host
        port (str): port
        sgbd (str, optional): Defaults to 'postgresql'. Change the sgbd string in the db connection-string

    Returns:
        sql alchemy db engine: 
    """ 
    con_string = string_connection(database, user, password, host, port, sgbd = 'postgresql')
    ## Creating db string connection
    return create_engine(con_string)

def create_string_connection(database: str, user: str, password: str, host: str, port: str, sgbd = 'postgresql'):
    """
    Declare a db connection

    Args:
        database (str): database name
        user (str): user
        password (str): password
        host (str): host
        port (str): port
        sgbd (str, optional): Defaults to 'postgresql'. Change the sgbd string in the db connection-string

    Returns:
        connection string: 
    """    

    ## Creating db string connection
    if sgbd == 'postgresql':
        con_string = f'postgresql://{user}:{password}@{host}:{port}/{database}'
    if sgbd == 'mysql':
        con_string = f'mysql+pymysql://{user}:{password}@{host}:{port}/{database}'
    if sgbd == 'redshift':
        con_string = f'redshift+psycopg2://{user}:{password}@{host}:{port}/{database}'
    return con_string